
class LocalNavigationSupport:
    """Mixin-class for next/up/previous support"""
    def getNext(self,context):
        """next object as defined by the chapter_ids
        or contentIds or None"""
        ids = getattr(self,'getChapter_ids','contentIds')()
        ids = list(ids)
        current_index = ids.index(context.getId())
        try:
            next_id = ids[current_index+1]
        except IndexError:
            return None
        return getattr(self,next_id,None)

    def hasNext(self,context):
        """True if there is a next chapter, False otherwise """
        return self.getNext(context) and True or False

    def nextTitle(self,context):
        """title of the next object """
        try:
            return self.getNext(context).Title()
        except AttributeError:
            return None

    def nextUrl(self,context,scale=""):
        """url of the next object"""
        query_string = ""
        if scale:
            query_string += "?image_scale=%s" % scale
        try:
            return self.getNext(context).absolute_url() + query_string
        except AttributeError:
            return None

    def getPrevious(self,context):
        """previous object as defined by the chapter_ids
        or contentIds or None"""
        ids = getattr(self,'getChapter_ids','contentIds')()
        ids = list(ids)
        current_index = ids.index(context.getId())
        if current_index == 0:
            return None
        next_id = ids[current_index-1]
        return getattr(self,next_id,None)

    def hasPrevious(self,context):
        """True if there is a previous chapter; False otherwise"""
        return self.getPrevious(context) and True or False

    def previousTitle(self,context):
        """title of the previous object """
        try:
            return self.getPrevious(context).Title()
        except AttributeError:
            return None

    def previousUrl(self,context,scale=""):
        """url of the previous object"""
        query_string = ""
        if scale:
            query_string += "?image_scale=%s" % scale
        try:
            return self.getPrevious(context).absolute_url() + query_string
        except AttributeError:
            return None

   
