import re, os, tempfile
from DateTime import DateTime
from Globals import package_home
from AccessControl import ClassSecurityInfo
from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.CMFCorePermissions import View
from Products.CMFCore.utils import getToolByName
from Products.Archetypes.public import *

from config import GLOBALS, BOOK_TITLE, BOOK_DESCRIPTION,\
     CHAPTER_IDS, CHAPTER_TITLES, CHAPTER_DESCRIPTIONS
from LocalNavigationSupport import LocalNavigationSupport

PACKAGE_HOME = package_home(GLOBALS)

image_formats = ('gif','jpg','jpeg','png')

MyBaseSchema = BaseSchema.copy()
MyBaseSchema['title'].default = BOOK_TITLE
MyBaseSchema['description'].default = BOOK_DESCRIPTION


schema = MyBaseSchema + Schema((
    LinesField('chapter_ids',
               default=CHAPTER_IDS),
    StringField('source',
                schemata='upload',
                vocabulary=('cvs','local','fallback'),
                default='fallback',
                widget=SelectionWidget()
                ),
    StringField('cvs_command',
                schemata='upload',
                default='cvs -z3 -d:pserver:anonymous@cvs.sourceforge.net:/cvsroot/plone-docs co PloneBook',
                ),
    StringField('local_path',
                schemata='upload',
                ),
    ))

class PloneBook(BaseFolder,LocalNavigationSupport):
    """special purpose content type to manage an online version
    of Andy McKay's Definitive Guide to Plone
    """
    portal_type = archetype_name = "Plone Book"
    global_allow = 0
    filter_content_types = 1
    allowed_content_types = ['Chapter','Folder','Image','Document','File']

    security       = ClassSecurityInfo()
    
    schema = schema

    actions = (
        {
        'id'           : 'view',
        'name'         : 'View',
        'action'       : 'string:${object_url}/plone_book_view',
        'permissions'  : (View,),
        'category'     : 'object',
        },
        {
        'id'           : 'update',
        'name'         : 'Update',
        'action'       : 'string:${object_url}/update_form',
        'permissions'  : (CMFCorePermissions.ModifyPortalContent,),
        'category'     : 'object',
        },
        )


    # for the update support
    # (via cvs at SourceForge or local checkout or fallback

    security.declareProtected(CMFCorePermissions.ModifyPortalContent,
                              'update')
    def update(self,source='cvs',local_path='',cvs_command=''):
        """main method to be called for uploads and updates"""
        path = self._getSourceFolderPath(source,local_path,cvs_command)
        self.updateChapters(path)
        self.updateImages(path)
        detail = local_path + cvs_command
        self._updateHistory(source,detail)
        if source == 'cvs':
            self._cleanup(source,path)

    security.declareProtected(CMFCorePermissions.ModifyPortalContent,
                              'updateChapters')
    def updateChapters(self,source_path):
        for id in self.getChapter_ids():
            body = open(os.path.join(source_path,id),'r').read()
            body = self.urlify(body)
            body = self.extendImageDirective(body)
            self._updateChapter(id,body)

    def _updateChapter(self,id,body):
        if id not in self.contentIds():
            self.invokeFactory(type_name='Chapter',
                               id=id)
            chapter = getattr(self,id)
            chapter.setTitle(CHAPTER_TITLES.get(id,id))
            chapter.setDescription(CHAPTER_DESCRIPTIONS.get(id,' '))
            chapter.manage_addProperty('left_slots',
                                       (),
                                       'lines',
                                       )
            chapter.manage_addProperty('right_slots',
                                       (),
                                       'lines',
                                       )
        chapter = getattr(self,id)
        chapter.setText(body,encoding='utf-8')
        chapter.setContentType('text/restructured')

    def updateImages(self,source_path):
        ids = os.listdir(os.path.join(source_path,'img'))
        for id in ids:
            if id.split('.')[-1] in image_formats:
                data = open(os.path.join(source_path,'img',id),'rb').read()
                self._updateImage(id,data)

    def _updateImage(self,id,data):
        folder = self._getImageFolder()
        if id not in folder.contentIds():
            folder.invokeFactory(type_name='Image',
                                 id=id)
        image = getattr(folder,id)
        try:
            image.update_data(data)
        except AttributeError:
            # we most probably have ATCT's Image type
            image.setImage(data)

    def _getImageFolder(self):
        if 'img' not in self.contentIds():
            self.invokeFactory(type_name='Folder',
                               id='img')
            img = getattr(self,'img')
            img.manage_addProperty('left_slots',
                                   (),
                                   'lines',
                                   )
            img.manage_addProperty('right_slots',
                                   (),
                                   'lines',
                                   )
        return getattr(self,'img')                
                                   
    def _getSourceFolderPath(self,source,source_path,cvs_command):
        if source == 'cvs':
            return self._getCVSCheckOut(cvs_command)
        if source == 'local':
            return source_path or self.getLocal_path()
        return os.path.join(PACKAGE_HOME,'content')

    def _getCVSCheckOut(self,command=''):
        if not command:
            command = self.getCvs_command()
        wd = tempfile.mkdtemp()
        print "Got %s as temp dir" % wd
        os.system("cd %s; %s" % (wd,command))
        
        return os.path.join(wd,'PloneBook','en')

    def _cleanup(self,source,path):
        if not source == 'cvs':
            return None
        for root, dirs, files in os.walk(path, topdown=False):
            for name in files:
                os.remove(os.path.join(root, name))
            for name in dirs:
                os.rmdir(os.path.join(root, name))
        

    # for the update history
    # nested tuple with elements
    # 0: user
    # 1: time
    # 2: source
    # 3: detail
    def _updateHistory(self,source,detail=''):
        if not hasattr(self,'_upload_history'):
            self._upload_history = ()
        self._upload_history += ([self._currentUser(),
                                  DateTime(),
                                  source,
                                  detail],)
    def getUploadHistory(self):
        """the entire history as nested tuple with elements
        0: user, 1: time, 2: source, 3: detail
        """
        return getattr(self,'_upload_history',None)

    def getLastUpdate(self):
        """last entry from the upload_history"""
        history = self.getUploadHistory()
        return history and history[-1] or None

    def _currentUser(self):
        mtool = getToolByName(self,'portal_membership')
        return mtool.getAuthenticatedMember().getId()

    # final tweaks to improve the rendered output

    # URLify (just remove the formatting * from the URLs)

    def urlify(self, strg):
        """ add default protocol and remove formatting stars """
        strg = strg.replace(' www.',' http://www.')
        strg = re.sub(r'\*(?P<url>http://[^*]+)\*', '\g<url>', strg)
        return strg.strip()

    def extendImageDirective(self, text):
        """add a class and target declaration to the image
        directives"""
        text = re.sub(r'.. image:: img/(?P<img_id>[\S]+)',
                      '.. image:: img/\g<img_id>\n    :class: original\n    :target: img/\g<img_id>/image_view_fullscreen',
                      text)
        return text

    def scaleImages(self, source, scale='original'):
        """modify the 'class' statement on the image directives
        if not 'original'
        """
        if scale == 'original': return source
        return source.replace('class="original"','class="%s"'%scale)

registerType(PloneBook)
